/**
 * Package defining the APIs for the data interfaces for OAS
 */
package com.github.ansell.oas.data;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;

import org.openrdf.model.BNode;
import org.openrdf.model.Literal;
import org.openrdf.model.Model;
import org.openrdf.model.URI;
import org.restlet.security.User;

import com.github.ansell.oas.objects.Annotation;
import com.github.ansell.oas.objects.Ontology;
import com.github.ansell.oas.objects.ResourceCount;
import com.github.ansell.oas.objects.SearchResult;
import com.github.ansell.oas.webservice.OasAction;

/**
 * A data interface, enabling abstract access to the datastore.
 * 
 * In testing, this may be an in-memory datastore, but in production it should be a database.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public interface DataHandler
{
    /**
     * Store AuditLog details for a request.
     * 
     * @param message
     * @param referenceUri
     * @param action
     * @param currentUser
     * @param ipAddress
     */
    void addLogDetailsForRequest(String message, String referenceUri, OasAction action, User currentUser,
            String ipAddress);
    
    /**
     * Creates a new Blank Node. Blank Node identifiers are transient and are not referenceable or
     * stable, even inside of an RDF database, so there is no customisation available.
     * 
     * @return A new Blank Node that can then be used as a Resource wherever Resources are allowed.
     */
    BNode createBlankNode();
    
    /**
     * Creates a plain literal based on the given string.
     * 
     * @param plainLiteralString
     *            A literal string to create a new plain literal.
     * @return A plain literal representing the given string.
     */
    Literal createLiteral(String plainLiteralString);
    
    /**
     * Creates a language literal based on languageLiteralString, with the language annotation given
     * by languageCode.
     * 
     * @param languageLiteralString
     *            The string value of the literal to be created.
     * @param languageCode
     *            The language that is to be assigned to the new literal.
     * @return A literal which contains a language annotation based on languageCode and whose value
     *         is based on languageLiteralString
     */
    Literal createLiteral(String languageLiteralString, String languageCode);
    
    /**
     * Creates a typed literal based on the given string and the datatype given in the URI.
     * 
     * @param typedLiteralString
     *            The string to use to create a typed literal.
     * @param datatypeUri
     *            The datatype to use to create the typed literal.
     * @return A typed literal, where the value of the literal is the string, and the datatype is
     *         the URI.
     */
    Literal createLiteral(String typedLiteralString, URI datatypeUri);
    
    /**
     * Get a URI object directly based on the string URI.
     * 
     * @param uri
     *            A string that represents a URI.
     * @return A URI instance created based on the given string URI.
     */
    URI createURI(String uri);
    
    /**
     * Instruct the data handler to delete the annotation with the given URI.
     * 
     * @param annotationIdUri
     *            A URI that is assigned to an annotation which is to be deleted
     * @return true if the annotation was successfully deleted, and false otherwise.
     */
    boolean deleteAnnotation(URI annotationIdUri);
    
    /**
     * Deletes all versions of the ontology with the given ontology URI.
     * 
     * @param nextOntologyUri
     *            The URI of the ontology to delete all versions of.
     * @return True if the ontology and all of its versions were deleted and false if the ontology
     *         was not recognised as being managed.
     */
    boolean deleteOntologyAllVersions(URI nextOntologyUri);
    
    /**
     * Deletes the version of the ontology matching the given Ontology object.
     * 
     * @param nextOntology
     *            The Ontology metadata object containing the ontology and version URI for the
     *            ontology to be deleted.
     * @return True if the ontology was deleted and false otherwise.
     */
    boolean deleteOntologyVersion(Ontology nextOntology);
    
    /**
     * Streams the contents of the given ontology to the given output stream, using the given MIME
     * type as the file format.
     * 
     * @param nextOntology
     *            The ontology to download.
     * @param outputStream
     *            The output stream to stream the ontology contents to.
     * @param mimeType
     *            The desired MIME type for the downloaded ontology.
     */
    void downloadOntology(Ontology nextOntology, OutputStream outputStream, String mimeType);
    
    /**
     * Streams a partial contents of the given ontology to the given output stream, using the given
     * MIME type as the file format.
     * 
     * NOTE: Inferences are always taken into account when resolving this method, as the subset by
     * definition includes the inferences.
     * 
     * @param nextOntology
     *            The ontology to download.
     * @param outputStream
     *            The output stream to stream the ontology contents to.
     * @param mimeType
     *            The desired MIME type for the downloaded ontology.
     * @param parentUri
     *            The starting point for the property path used to derive the subset. If this is
     *            null, then owl:Thing will be used as the parent.
     * @param recursive
     *            Whether the subset should be recursive.
     * @param addCounts
     *            TODO
     */
    void downloadOntologySubset(Ontology nextOntology, ByteArrayOutputStream outputStream, String mimeType,
            URI parentUri, boolean recursive, boolean addCounts);
    
    /**
     * Streams the contents of the given ontology to the given output stream, using the given MIME
     * type as the file format, including inferences.
     * 
     * @param nextOntology
     *            The ontology to download.
     * @param outputStream
     *            The output stream to stream the ontology contents to.
     * @param mimeType
     *            The desired MIME type for the downloaded ontology.
     */
    void downloadOntologyWithInferences(Ontology nextOntology, OutputStream outputStream, String mimeType);
    
    /**
     * Instruct the data handler to retrieve the given annotation using the given URI.
     * 
     * @param annotationIdUri
     *            The URI to use to locate the required annotation.
     * @return An annotation if one was available, or null otherwise.
     */
    Annotation getAnnotation(URI annotationIdUri);
    
    ResourceCount getAnnotationCountsByAnnotatedObjectUri(URI objectUriResource);
    
    /**
     * Instruct the data handler to retrieve counts of annotations where the annotation references
     * the given ontology term URI.
     * 
     * @param ontologyTermUri
     *            The ontology term to search for annotations with.
     * @param nextOntology
     *            TODO
     * @param recursive
     *            Whether to include annotation counts for children of the given ontology term
     * @return A count of the number of annotations that reference the given ontology term URI.
     */
    ResourceCount getAnnotationCountsByOntologyTermUri(URI ontologyTermUri, Ontology nextOntology, boolean recursive);
    
    /**
     * Instruct the data handler to retrieve all of the annotations that referenced the given object
     * type.
     * 
     * @param annotatedObjectType
     *            The object type to search for in the annotation set.
     * @return A collection of annotations that reference the given object type
     */
    Collection<Annotation> getAnnotationsByAnnotatedObjectType(URI annotatedObjectType);
    
    /**
     * Instruct the data handler to retrieve all of the annotations that referenced the given object
     * URI.
     * 
     * @param annotatedObjectUri
     *            The object URI to search for in the annotation set.
     * @return A collection of annotations that reference the given object URI
     */
    Collection<Annotation> getAnnotationsByAnnotatedObjectUri(URI annotatedObjectUri);
    
    /**
     * Instruct the data handler to retrieve all of the annotations that referenced the given
     * ontology term URI.
     * 
     * @param ontologyTermUri
     *            The ontology term URI to search for in the annotation set.
     * @return A collection of annotations that reference the given ontology term URI
     */
    Collection<Annotation> getAnnotationsByOntologyTermUri(URI ontologyTermUri);
    
    /**
     * Finds the children of the given parent URI in the given ontology, optionally performing a
     * recursive search for children below the item.
     * 
     * @param ontology
     *            The ontology URI, and version to use to determine the children of the given parent
     *            URI.
     * @param parentUri
     *            The parent URI to get children for, or null to start at the top elements in the
     *            ontology.
     * @param recursive
     *            If recursive is true, then the algorithm will attempt to find more than a single
     *            layer of children.
     * @param limit
     *            The limit of children to return for each result.
     * @param offset
     *            The offset within the set of children results to use.
     * @return A closeable iteration containing the URIs of children for the given parent URI.
     */
    Map<URI, Collection<URI>> getChildren(Ontology ontology, URI parentUri, boolean recursive, int limit, int offset);
    
    /**
     * Counts the children of the given parent URI in the given ontology, optionally performing a
     * recursive search for children below the item.
     * 
     * @param ontology
     *            The ontology URI, and version to use to determine the children of the given parent
     *            URI.
     * @param parentUri
     *            The parent URI to get children for.
     * @param recursive
     *            If recursive is true, then the algorithm will attempt to find more than a single
     *            layer of children.
     * @return The count of children below the parent URI
     */
    ConcurrentMap<URI, ResourceCount> getChildrenCount(Ontology ontology, List<URI> parentUri, boolean recursive);
    
    /**
     * Returns an ordered list of labels for the URI in the context of the given ontology.
     * 
     * @param nextOntology
     *            The ontology to search for the label in.
     * @param nextChildUri
     *            The URI to find a label for
     * @return A list containing labels ordered by preference where possible.
     */
    List<Literal> getLabelsForUri(Ontology nextOntology, URI nextChildUri);
    
    /**
     * Find the latest ontology that matches a given ontologyUri.
     * 
     * NOTE: If the URI identifies a specific version for an ontology, that version of the ontology
     * is returned. The latest/current semantics only affects queries based on the Ontology IRI.
     * 
     * @param ontologyUri
     *            The ontology URI to search for in the database.
     * @return The latest ontology that matches the given ontologyUri, or null if no ontologies
     *         match that URI.
     */
    Ontology getLatestOntology(URI ontologyUri);
    
    Model getLogDetails(int offset, int limit);
    
    /**
     * Gets a range of ontologies starting from the given start point and continuing to the given
     * offset based on the algorithm implemented by this data handler for sorting.
     * 
     * A data handler should iterate through the most current versions of accessible ontologies to
     * return each exactly once, assuming that the set of ontologies are not modified during the
     * iteration process.
     * 
     * If the set of accessible ontologies are modified between calls, the results of successive
     * calls may not be related.
     * 
     * @param offset
     *            The start offset. Starts at 0 and continues to accessibleOntologies.size()-1.
     * @param limit
     *            The maximum number of results to return.
     * @return A list of ontologies from the given start offset to a maximum of limit results.
     */
    List<Ontology> getOntologies(int offset, int limit);
    
    /**
     * Return a list of ontology versions created using the given ontology URI.
     * 
     * @param ontologyUri
     *            The ontology URI to search for in the database.
     * @return A collection of ontologies that match the given ontology URI.
     */
    List<Ontology> getOntologiesByUri(URI ontologyUri);
    
    /**
     * Find the latest ontology that matches a given ontologyUri.
     * 
     * @param ontologyUri
     *            The ontology URI to search for in the database.
     * @param ontologyVersionUri
     *            The managed ontology version URI to search for in connection with the given
     *            ontology URI.
     * @return The ontology with the given ontology URI and managed ontology version URI, or null if
     *         no matching ontologies were found.
     */
    Ontology getVersionedOntology(URI ontologyUri, URI ontologyVersionUri);
    
    /**
     * Returns a list of annotations that are sorted using some internal criteria and returns a
     * subset of the sorted annotations between the given offset and limit.
     * 
     * @param limit
     *            The maximum number of annotations to return.
     * @param offset
     *            The offset in the sorted list of annotations.
     * @return
     */
    Collection<Annotation> listAnnotations(int limit, int offset);
    
    /**
     * Searches across all ontologies for the given search term.
     * 
     * @param searchTerm
     * @param limit
     * @param offset
     * @return A list of search results.
     */
    List<SearchResult> searchAllOntologies(String searchTerm, int limit, int offset);
    
    /**
     * Searches across annotations for the given search term.
     * 
     * @param searchTerm
     * @param limit
     * @param offset
     * @return
     */
    Collection<Annotation> searchAnnotations(String searchTerm, int limit, int offset);
    
    /**
     * Searches across the given ontologies for the given search term.
     * 
     * @param ontologies
     * @param searchTerm
     * @param limit
     * @param offset
     * @return A list of search results.
     */
    List<SearchResult> searchOntologies(Collection<Ontology> ontologies, String searchTerm, int limit, int offset);
    
    /**
     * Searches across ontology labels and descriptions for the given search term.
     * 
     * @param searchTerm
     * @param limit
     * @param offset
     * @return A list of result {@link Ontology} items.
     */
    List<SearchResult> searchOntologyLabels(String searchTerm, int limit, int offset);
    
    /**
     * Instruct the data handler to store the given annotation, including a valid URI in the
     * annotation.
     * 
     * @param nextAnnotation
     *            The annotation to store, including a valid unique URI.
     * @return true if the annotation was stored, and false otherwise.
     */
    boolean storeAnnotation(Annotation nextAnnotation);
    
    /**
     * Instruct the data handler to store the given ontology, updating the version details inside of
     * the ontology as necessary, and returning the resulting ontology.
     * 
     * @param nextOntology
     *            The ontology metadata to store.
     * @param inputStream
     *            The inputStream to use to import the contents of the ontology from.
     * @param baseUri
     *            The Base URI to use when processing the document.
     * @param mimeType
     *            The MIME type to use to interpret the input on the input stream.
     * @param deleteOldVersions
     *            If true, old versions of the ontology will be removed after this ontology is
     *            stored.
     * @return The resulting ontology object. To download the actual ontology contents call
     *         downloadOntology using the resulting ontology object.
     */
    Ontology storeOntology(Ontology nextOntology, InputStream inputStream, String baseUri, String mimeType,
            boolean deleteOldVersions);
    
    /**
     * Instruct the data handler to store the given ontology, updating the version details inside of
     * the ontology as necessary, and returning the resulting ontology.
     * 
     * @param nextOntology
     *            The ontology metadata to store.
     * @param inputStream
     *            The inputStream to use to import the contents of the ontology from.
     * @param baseUri
     *            The Base URI to use when processing the document.
     * @param mimeType
     *            The MIME type to use to interpret the input on the input stream.
     * @param deleteOldVersions
     *            If true, old versions of the ontology will be removed after this ontology is
     *            stored.
     * @return The resulting ontology object. To download the actual ontology contents call
     *         downloadOntology using the resulting ontology object.
     */
    Ontology storeOntology(Ontology nextOntology, Reader inputStream, String baseUri, String mimeType,
            boolean deleteOldVersions);
    
    /**
     * Instruct the data handler to update the given annotation, using the unique ID URI that is
     * given inside of the annotation.
     * 
     * @param nextAnnotation
     *            The annotation, including a valid unique URI, that is to be updated.
     * @return true if the annotation was updated, and false otherwise
     */
    boolean updateAnnotation(Annotation nextAnnotation);
    
    Ontology setLatestOntology(URI ontologyUri, URI versionUri);
    
}
