/**
 * The package defining the interfaces for objects that are used by the OAS web service
 */
package com.github.ansell.oas.objects;

import org.openrdf.model.Literal;
import org.openrdf.model.Model;
import org.openrdf.model.URI;

/**
 * Defines the interface for the simple annotations that we are using in the ontology annotation
 * service. The annotation may not contain all of the given fields, especially in cases where the
 * annotation is used in a list to be sent through a web service. In these cases for performance
 * reasons, only the relevant fields may be filled.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public interface Annotation
{
    /**
     * Retrieves the type, represented by a URI, for the object that is annotated by this
     * annotation.
     * 
     * @return The URI representing the type of the object being annotated by this annotation
     */
    URI getAnnotatedObjectType();
    
    /**
     * Retrieves the URI of the object that is annotated by this annotation.
     * 
     * @return The URI of the object that is annotated by this annotation.
     */
    URI getAnnotatedObjectUri();
    
    /**
     * Returns the URI that defines the unique identifier for this annotation.
     * 
     * @return The unique identifier URI for this annotation.
     */
    URI getAnnotationUniqueId();
    
    /**
     * Retrieves a URI designating the person or agent who created this annotation.
     * 
     * @return The URI designating the person or agent who created this annotation.
     */
    URI getAnnotator();
    
    /**
     * 
     * @return A human-readable label for the person or agent who created this annotation.
     */
    String getAnnotatorLabel();
    
    /**
     * Retrieves the URI representing the term in an ontology that is being used to describe the
     * meaning for this annotation.
     * 
     * @return The URI from an ontology that is being used to describe the meaning for this
     *         annotation.
     */
    URI getOntologyTermUri();
    
    /**
     * Retrieves the URI of the ontology that {@link #getOntologyTermUri()} is defined in. This URI
     * may be the URI of the OWL Version IRI for the ontology, specifying that the annotation is
     * specifically attached to that version of the ontology.
     * 
     * @return The URI from an ontology that is being used to describe the meaning for this
     *         annotation.
     */
    URI getOntologyUri();
    
    /**
     * Retrieves the literal used to give a human readable description to this annotation.
     * 
     * @return The Literal used to describe this annotation.
     */
    Literal getTag();
    
    /**
     * Sets the URI representing the type of the object being annotated by this annotation.
     * 
     * @param annotatedObjectType
     *            The URI representing the type of the object being annotated.
     */
    void setAnnotatedObjectType(URI annotatedObjectType);
    
    /**
     * Sets the URI for the object that is annotated by this annotation.
     * 
     * @param annotatedObjectUri
     *            The URI for the object that is annotated by this annotation.
     */
    void setAnnotatedObjectUri(URI annotatedObjectUri);
    
    /**
     * Sets the URI that will be used to define the annotation ID for this annotation.
     * 
     * @param annotationUniqueId
     *            Sets the unique identifier for this annotation.
     */
    void setAnnotationUniqueId(URI annotationUniqueId);
    
    /**
     * Sets the person or agent who created this annotation based on the given URI.
     * 
     * @param annotator
     *            The URI designating the person or agent who created this annotation.
     */
    void setAnnotator(URI annotator);
    
    /**
     * 
     * @param annotatorLabel
     *            A label for the person or agent who created this annotation.
     */
    void setAnnotatorLabel(String annotatorLabel);
    
    /**
     * Sets the URI representing the term in an ontology that is being used to describe the meaning
     * for this annotation.
     * 
     * @param ontologyTermUri
     *            The URI from an ontology to be used for the meaning of this annotation.
     */
    void setOntologyTermUri(URI ontologyTermUri);
    
    /**
     * Sets the URI representing the ontology that contains {@link #getOntologyTermUri()}, which may
     * be an OWL Version IRI if the user wishes to specify that the annotation be attached to a
     * particular version of the ontology.
     * 
     * @param ontologyUri
     *            The URI of an ontology to be used for the meaning of this annotation.
     */
    void setOntologyUri(URI ontologyUri);
    
    /**
     * Sets the human readable description for this annotation to the given literal.
     * 
     * @param tag
     *            The human readable description for this annotation, in the form of a literal.
     */
    void setTag(Literal tag);
    
    /**
     * Convert the annotation to RDF statements.
     * 
     * @return
     */
    Model toRdf();
}