/**
 * The package defining the interfaces for objects that are used by the OAS web service
 */
package com.github.ansell.oas.objects;

import java.util.List;

import org.openrdf.model.Model;
import org.openrdf.model.URI;
import org.openrdf.repository.Repository;
import org.semanticweb.owlapi.model.OWLOntologyID;
import org.semanticweb.owlapi.profiles.OWLProfile;

/**
 * Defines the interface for all ontologies stored by the ontology management services.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public interface Ontology extends Comparable<Ontology>
{
    /**
     * Check the uniqueness of, and update if necessary, the current ontology version URI based on
     * the versions that are currently in the given repository.
     * 
     * The URI that the ontology manager will use to identify this version, may be different to the
     * discovered ontology version URI if the discovered ontology version URI did not change between
     * submissions of different ontologies.
     * 
     * <br />
     * 
     * The manager assigns this property internally based on the uniqueness of the key returned
     * using the results from getOntologyUri and getOntologyVersionUri.
     * 
     * <br />
     * 
     * If getOntologyUri is already known, and getOntologyVersionUri is unique, then this property
     * is set to the result from getOntologyVersionUri.
     * 
     * <br />
     * 
     * If getOntologyUri is unknown, and getOntologyVersionUri is set to a valid URI, then this
     * property is set to the result from getOntologyVersionUri.
     * 
     * <br />
     * 
     * If getOntologyUri is already known, and getOntologyVersionUri is not unique, then this
     * property is set to a new, locally unique, URI that may not be syntactically or structurally
     * related to getOntologyUri or getOntologyVersionUri.
     * 
     * <br />
     * 
     * If getOntologyUri is unknown, and getOntologyVersionUri is not set to a valid URI, then this
     * property is set to a new, locally unique, URI that may not be syntactically or structurally
     * related to getOntologyUri.
     * 
     * <br />
     * 
     * If getOntologyUri is not a valid URI then this method throws an IllegalStateException.
     * 
     * <br />
     * 
     * This method uses the given ontologyManagerRepository along with the given
     * ontologyManagerGraphURI to determine whether the managed ontology version URI is valid when
     * the method is first called.
     * 
     * <br />
     * 
     * In addition, this version of the method allows users to force validation for this call, or by
     * comparison, enforce no validation, at the risk of overwriting previous ontology data and
     * metadata.
     * 
     * @param ontologyManagerRepository
     *            The repository to search for ontology management information in.
     * @param forceValidationOfLocalVersionAgainstRepository
     *            Tell the algorithm to always try to validate the local ontology version URI
     *            against the repository, even if it is set to a non-null value.
     * @param useLocalVersionIfSet
     *            Tell the algorithm to use the local version if it is set, as long as
     *            validateLocalVersionAgainstRepository is false.
     * @param contexts
     *            The contexts to search in the given repository for ontology management
     *            information.
     * @return True if the version URI was changed as a result of this method being called, and
     *         false otherwise.
     */
    boolean ensureVersionUriUniqueness(Repository ontologyManagerRepository,
            boolean forceValidationOfLocalVersionAgainstRepository, boolean useLocalVersionIfSet, URI... contexts);
    
    /**
     * The total number of concrete axioms.
     * 
     * @return Concrete Axiom Count
     */
    int getConcreteAxiomCount();
    
    /**
     * The total number of logical concrete axioms.
     * 
     * @return Logical Axiom Count
     */
    int getConcreteLogicalAxiomCount();
    
    /**
     * Provides simple access to the combination of URIs that need to be searched for this ontology.
     * 
     * This list includes the ontology version URI and the inferred ontology URI.
     * 
     * @return An array of context URIs that can be used with the Sesame contexts parameter.
     * @throws IllegalStateException
     *             If the ontology version URI is not currently setup.
     */
    URI[] getContextUris();
    
    /**
     * 
     * @return A string containing a pattern that can be used in the WHERE clause of a SPARQL
     *         construct query with the following construct pattern: "CONSTRUCT { ?parent
     *         <http://purl.org/oas/ontology#hasChild> ?child . } WHERE {
     *         ${getRecursiveChildQuery()} }"
     * 
     */
    String getDirectChildQuery();
    
    /**
     * The number of individuals in the ontology, excluding the imports closure.
     * 
     * @return Number of Individuals
     */
    int getIndividualCount();
    
    /**
     * The total number of inferred axioms.
     * 
     * @return Inferred Axiom Count
     */
    int getInferredAxiomCount();
    
    /**
     * Gets the URI of the ontology containing inferred axioms for this version of this ontology.
     * 
     * @return The current URI of the ontology containing inferred axioms for this version of this
     *         ontology, or null if it is not currently known.
     */
    URI getInferredOntologyUri();
    
    /**
     * The standard, shared, URI for this Ontology. This URI must not change between versions, or
     * the manager will not understand the relationship between different versions.
     * 
     * @return The standard, shared, URI for this Ontology.
     */
    URI getOntologyUri();
    
    /**
     * The shared version URI for this Ontology. This URI should change between different versions.
     * If a user does not change between successive versions, the manager will override the declared
     * version with a locally unique managed version URI.
     * 
     * @return The shared version URI, should be assigned using the versionIRI predicate in OWL-2.
     */
    URI getOntologyVersionUri();
    
    /**
     * 
     * @return A collection of profiles that this version of this ontology was determined to be in.
     */
    List<OWLProfile> getProfiles();
    
    /**
     * 
     * @return A string containing a pattern that can be used in the WHERE clause of a SPARQL
     *         construct query with the following construct pattern: "CONSTRUCT { ?parent
     *         <http://purl.org/oas/ontology#hasChild> ?child . } WHERE {
     *         ${getRecursiveChildQuery()} }"
     * 
     */
    String getRecursiveChildQuery();
    
    /**
     * The total number of concrete axioms.
     * 
     * @param axiomCount
     */
    void setConcreteAxiomCount(int axiomCount);
    
    /**
     * The total number of logical concrete axioms.
     * 
     * @param logicalAxiomCount
     */
    void setConcreteLogicalAxiomCount(int logicalAxiomCount);
    
    /**
     * A construct where pattern that can be used to populate triples of the following form: ?parent
     * <http://purl.org/oas/ontology#hasChild> ?child .
     * 
     * @param nextChildSearchQueryPattern
     */
    void setDirectChildQuery(String nextChildSearchQueryPattern);
    
    /**
     * The number of individuals in the ontology, excluding the imports closure.
     * 
     * @param individualCount
     */
    void setIndividualCount(int individualCount);
    
    /**
     * The total number of inferred axioms.
     * 
     * @param axiomCount
     */
    void setInferredAxiomCount(int axiomCount);
    
    /**
     * Sets the URI of the ontology containing inferred axioms for this version of this ontology.
     * 
     * @param inferredOntologyVersionUri
     *            The current URI of the ontology containing inferred axioms for this version of
     *            this ontology.
     */
    void setInferredOntologyUri(URI inferredOntologyUri);
    
    /**
     * Sets the details based on RDF Triples representing this ontology using the given collection
     * of OpenRDF Statements.
     * 
     * <br />
     * 
     * The statements may contain the ontology URI, and the ontology version URI using the
     * versionIRI predicate.
     * 
     * <br />
     * 
     * If the ontology URI and/or the ontology version URI are not explicitly set prior to using
     * this method, the manager will attempt to populate them on a best-effort basis.
     * 
     * <br />
     * 
     * Note: This method does not set the actual contents of the ontology, as it may be too large to
     * be saved in memory, and hence should only every be streamed to a database on disk and then
     * queried.
     * 
     * @param ontologyDetails
     *            The statements representing this ontology.
     */
    void setOntologyDetails(Model ontologyDetails);
    
    /**
     * Sets the standard, shared, URI for this ontology. This URI should not change between
     * different versions of a single ontology.
     * 
     * @param ontologyUri
     *            The standard, shared, URI for this Ontology.
     */
    void setOntologyUri(URI ontologyUri);
    
    /**
     * The shared version URI for this Ontology. This URI should change between different versions.
     * If it does not change between successive versions, the manager will not use this property
     * directly, and will use a locally unique managed version URI. However, this process requires a
     * call to checkVersionUriUniqueness after the version is set.
     * 
     * @param ontologyVersionUri
     *            The shared version URI, should be assigned using the versionIRI predicate in
     *            OWL-2.
     */
    void setOntologyVersionUri(URI ontologyVersionUri);
    
    /**
     * 
     * @param matchingProfiles
     *            A collection of profiles that this ontology is known to be in.
     */
    void setProfiles(List<OWLProfile> matchingProfiles);
    
    /**
     * A construct where pattern that can be used to populate triples of the following form: ?parent
     * <http://purl.org/oas/ontology#hasChild> ?child .
     * 
     * @param nextChildSearchQueryPattern
     */
    void setRecursiveChildQuery(String nextChildSearchQueryPattern);
    
    /**
     * Convert this instance of Ontology to an instance of OWLOntologyID
     * 
     * @return An instance of OWLOntologyID that contains the equivalent information from this
     *         Ontology.
     */
    OWLOntologyID toOWLOntologyID();
}