/**
 * 
 */
package com.github.ansell.oas.webservice;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.restlet.security.Role;

import com.github.ansell.oas.data.DataHandlerConstants;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.propertyutil.PropertyUtil;
import com.github.ansell.restletutils.RestletUtilRoles;

/**
 * Provides constants to use in the authentication interface, including whether authentication is
 * required for a particular action and what the error message should be if a request fails.
 * 
 * FIXME: Roles are currently hardcoded into each action. Restlet-2.1 does not support anything
 * other than reference identity for equality between roles. Resltet-2.2 changes the roles equality
 * again, so this must be reviewed before updating the production version to Restlet-2.2.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public enum BaseOasActions implements OasAction
{
    /**
     * An action by a user asking to create a new ontology, or update an existing ontology.
     * 
     * By default both authenticated and admin users are allowed to create ontologies.
     */
    ONTOLOGY_CREATE(DataHandlerConstants.CREATE, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_ONTOLOGY_CREATE,
            OasProps.DEF_AUTH_ONTOLOGY_CREATE), PropertyUtil.getInstance().get(
            OasProps.PROP_ERROR_AUTH_ONTOLOGY_CREATE, OasProps.DEF_ERROR_AUTH_ONTOLOGY_CREATE), new HashSet<Role>(
            Arrays.asList(RestletUtilRoles.ADMIN.getRole(), RestletUtilRoles.AUTHENTICATED.getRole()))),
    
    /**
     * An action by a user asking to delete an ontology.
     * 
     * By default only authenticated admin users are allowed to delete ontologies.
     */
    ONTOLOGY_DELETE(DataHandlerConstants.DELETE, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_ONTOLOGY_DELETE,
            OasProps.DEF_AUTH_ONTOLOGY_DELETE), PropertyUtil.getInstance().get(
            OasProps.PROP_ERROR_AUTH_ONTOLOGY_DELETE, OasProps.DEF_ERROR_AUTH_ONTOLOGY_DELETE), Collections
            .singleton(RestletUtilRoles.ADMIN.getRole())),
    
    /**
     * An action by a user asking to read an ontology.
     * 
     * By default all unauthenticated users are allows to read ontologies.
     */
    ONTOLOGY_READ(DataHandlerConstants.GET, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_ONTOLOGY_READ,
            OasProps.DEF_AUTH_ONTOLOGY_READ), PropertyUtil.getInstance().get(OasProps.PROP_ERROR_AUTH_ONTOLOGY_READ,
            OasProps.DEF_ERROR_AUTH_ONTOLOGY_READ), Collections.<Role> emptySet()),
    
    /**
     * An action by an administrator asking to create a new user.
     * 
     * By default only admin users are allowed to create new users.
     */
    USER_CREATE(DataHandlerConstants.CREATE, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_USER_CREATE,
            OasProps.DEF_AUTH_USER_CREATE), PropertyUtil.getInstance().get(OasProps.PROP_ERROR_AUTH_USER_CREATE,
            OasProps.DEF_ERROR_AUTH_USER_CREATE), Collections.singleton(RestletUtilRoles.ADMIN.getRole())),
    
    /**
     * An action by a user asking to fetch a user
     * 
     * By default all authenticated and admin users can request get, although if they are not
     * admins, they will not be able to see information about other users.
     */
    USER_GET(DataHandlerConstants.GET, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_USER_GET,
            OasProps.DEF_AUTH_USER_GET), PropertyUtil.getInstance().get(OasProps.PROP_ERROR_AUTH_USER_GET,
            OasProps.DEF_ERROR_AUTH_USER_GET), new HashSet<Role>(Arrays.asList(RestletUtilRoles.ADMIN.getRole(),
            RestletUtilRoles.AUTHENTICATED.getRole()))),
    
    /**
     * An action by an administrator asking to delete an existing user.
     * 
     * By default only admin users are allowed to delete existing users.
     */
    USER_DELETE(DataHandlerConstants.DELETE, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_USER_DELETE,
            OasProps.DEF_AUTH_USER_DELETE), PropertyUtil.getInstance().get(OasProps.PROP_ERROR_AUTH_USER_DELETE,
            OasProps.DEF_ERROR_AUTH_USER_DELETE), Collections.singleton(RestletUtilRoles.ADMIN.getRole())),
    
    /**
     * An action by an administrator asking to edit the roles for a user.
     * 
     * By default only admin users are allowed to edit the roles for users.
     */
    ROLE_EDIT(DataHandlerConstants.UPDATE, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_ROLE_EDIT,
            OasProps.DEF_AUTH_ROLE_EDIT), PropertyUtil.getInstance().get(OasProps.PROP_ERROR_AUTH_ROLE_EDIT,
            OasProps.DEF_ERROR_AUTH_ROLE_EDIT), Collections.singleton(RestletUtilRoles.ADMIN.getRole())),
    
    /**
     * An action by a user asking to read an annotation.
     * 
     * By default all unauthenticated users are allows to read annotations.
     */
    ANNOTATION_READ(DataHandlerConstants.GET, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_ANNOTATION_READ,
            OasProps.DEF_AUTH_ANNOTATION_READ), PropertyUtil.getInstance().get(
            OasProps.PROP_ERROR_AUTH_ANNOTATION_READ, OasProps.DEF_ERROR_AUTH_ANNOTATION_READ), Collections
            .<Role> emptySet()),
    
    /**
     * An action by a user asking to create annotations.
     * 
     * By default all authenticated and admin users can create anontations.
     */
    ANNOTATION_CREATE(DataHandlerConstants.CREATE, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_ANNOTATION_CREATE,
            OasProps.DEF_AUTH_ANNOTATION_CREATE), PropertyUtil.getInstance().get(
            OasProps.PROP_ERROR_AUTH_ANNOTATION_CREATE, OasProps.DEF_ERROR_AUTH_ANNOTATION_CREATE), new HashSet<Role>(
            Arrays.asList(RestletUtilRoles.ADMIN.getRole(), RestletUtilRoles.AUTHENTICATED.getRole()))),
    
    /**
     * An action by a user asking to delete annotations.
     * 
     * By default only admin users can delete anontations.
     */
    ANNOTATION_DELETE(DataHandlerConstants.DELETE, PropertyUtil.getInstance().get(OasProps.PROP_AUTH_ANNOTATION_DELETE,
            OasProps.DEF_AUTH_ANNOTATION_DELETE), PropertyUtil.getInstance().get(
            OasProps.PROP_ERROR_AUTH_ANNOTATION_DELETE, OasProps.DEF_ERROR_AUTH_ANNOTATION_DELETE), new HashSet<Role>(
            Arrays.asList(RestletUtilRoles.ADMIN.getRole()))),
    
    /**
     * An action by a user requesting a pingback with their user identifier.
     * 
     * By default only authenticated users can use this service without receiving an error.
     */
    WHOAMI_CHECK(DataHandlerConstants.GET, "true", "Must be logged in to use this service", Collections
            .<Role> emptySet()),
    
    /**
     * An action by a user requesting to get access to the auditlog.
     * 
     * By default only admin users can get the auditlog.
     */
    AUDITLOG_GET(DataHandlerConstants.GET, "true", "Must be logged in to use this service", new HashSet<Role>(
            Arrays.asList(RestletUtilRoles.ADMIN.getRole())))
    
    ;
    
    private final DataHandlerConstants method;
    private final boolean authRequired;
    private final String errorMessage;
    private final Set<Role> roles;
    
    BaseOasActions(final DataHandlerConstants method, final String authenticationRequired, final String errorMessage,
            final Set<Role> roles)
    {
        this.method = method;
        this.authRequired = Boolean.valueOf(authenticationRequired);
        this.errorMessage = errorMessage;
        this.roles = roles;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.github.ansell.oas.webservice.OasActionInterface#getErrorMessage()
     */
    @Override
    public String getErrorMessage()
    {
        return this.errorMessage;
    }
    
    @Override
    public DataHandlerConstants getMethod()
    {
        return this.method;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.github.ansell.oas.webservice.OasActionInterface#isAuthRequired()
     */
    @Override
    public boolean isAuthRequired()
    {
        return this.authRequired;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see com.github.ansell.oas.webservice.OasActionInterface#isRoleRequired()
     */
    @Override
    public boolean isRoleRequired()
    {
        return !this.roles.isEmpty();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see
     * com.github.ansell.oas.webservice.OasActionInterface#matchesForRoles(java.util.Collection)
     */
    @Override
    public boolean matchesForRoles(final Collection<Role> authenticatedRoles)
    {
        if(!this.isRoleRequired())
        {
            return true;
        }
        
        for(final Role nextAuthenticatedRole : authenticatedRoles)
        {
            if(this.roles.contains(nextAuthenticatedRole))
            {
                return true;
            }
        }
        
        return false;
    }
}
