package com.github.ansell.oas.webservice;

import org.restlet.Application;
import org.restlet.Context;
import org.restlet.Request;
import org.restlet.Response;
import org.restlet.security.ChallengeAuthenticator;

import com.github.ansell.oas.data.DataHandler;
import com.github.ansell.propertyutil.PropertyUtil;
import com.github.ansell.restletutils.RestletUtilSesameRealm;

import freemarker.template.Configuration;

/**
 * The application which is the parent of all OAS Restlet resources. It provides the application
 * level functions for restlets including the DataHandler and the authentication implementation.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public abstract class OasWebServiceApplication extends Application
{
    public OasWebServiceApplication(Context context)
    {
        super(context);
    }
    
    /**
     * Checks whether the client is authenticated for the given action, and if they are not, the
     * response will have challenges inserted and the status will be set to HTTP 401.
     * 
     * Only call this method if authentication is needed for the given request.
     * 
     * @param authentication
     * @param request
     * @param response
     * @return True if the request is authenticated, and false otherwise.
     */
    public abstract boolean authenticate(OasAction authentication, Request request, Response response);
    
    /**
     * 
     * @return The ChallengeAuthenticator which is currently being used to respond to queries that
     *         require authentication.
     */
    public abstract ChallengeAuthenticator getAuthenticator();
    
    /**
     * Returns a data handler.
     * 
     * If no data handler has been specified it returns a new in-memory sesame data store using the
     * default constructor for SesameDataHandlerImpl.
     * 
     * @return An instance of the data handler interface that can be used to store, retrieve, and
     *         delete objects.
     */
    public abstract DataHandler getDataHandler();
    
    /**
     * 
     * @return The instance of {@link PropertyUtil} used by this application.
     */
    public abstract PropertyUtil getPropertyUtil();
    
    /**
     * Gets the realm which is used to manage users and roles.
     * 
     * @return
     */
    public abstract RestletUtilSesameRealm getRealm();
    
    /**
     * Returns the FreeMarker template configuration object for this application.
     * 
     * @return
     */
    public abstract Configuration getTemplateConfiguration();
    
    /**
     * 
     * @param auth
     *            The new ChallengeAuthenticator to be used to respond to queries which require
     *            authentication.
     */
    public abstract void setAuthenticator(ChallengeAuthenticator auth);
    
    /**
     * Set a data handler for use by this application.
     * 
     * @param nextDataHandler
     *            The dataHandler to set.
     */
    public abstract void setDataHandler(final DataHandler nextDataHandler);
    
    /**
     * Sets the realm which is used to manage users and roles.
     * 
     * @param nextRealm
     */
    public abstract void setRealm(RestletUtilSesameRealm nextRealm);
    
    /**
     * Set a new Freemarker Template Configuration for this application.
     * 
     * @param nextFreemarkerConfiguration
     */
    public abstract void setTemplateConfiguration(Configuration nextFreemarkerConfiguration);
}